;
; Include file CHAN defines a default channel map for RC640
;
;
; Ant   v0.1	12-11-2003			; dummy channel data
; Ant		v0.2	17-12-2003		; test chans
; Ant		v0.3	17-12-2003		; test chans
; Ant		v0.4	18-12-2003		; we understand the synth at last!
;                               		; a usable set of frequencies too.
; Ant   v0.5  08-01-2004    			; changed monitor flag, using head CTCSS encode
;                               		; blanked 50-99
;
; G4FPH	v0.6	11-05-2005			; Mods by G4FPH to include more channels
;
; Ant   v0.7    15-05-2005			; Tweak CTCSS settings & fix timeout
;
; Ant   v0.8    23-05-2005			; Tweak test channels
;                               
; Ant   v0.9    13-07-2005			; Add more test channels. Duplex flag leaves
;						; tx synth running in Rx rather than kill it
;
; Ant   v0.10   30-07-2005			; Added internet-link frequencies with Rx CTCSS
;
;        CPU     Z8601
;        INCLUDE STDDEFZ8.INC
;
SIGNATURE EQU    5DH              ;true value for signature byte
;
; Macro to generate BCD data from frequency
;
F5	MACRO	TXFREQ, RXFREQ
F_REF	SET	0.0125 ; 12.5kHz
M	SET	8      ; M=8
; Calculate RX divider
_RD	SET	INT(0.01 + (RXFREQ-10.7)/F_REF )
;	MESSAGE "RXFREQ " RXFREQ "DIV" RD
RN0	SET	_RD#M
RD1	SET	_RD/M
RN1	SET	RD1#10
RD2	SET	RD1/10
RN2	SET	RD2#10
RD3	SET	RD2/10
RN3	SET	RD3#10
RN4	SET	RD3/10
	IF	RN4>9
	ERROR	"Rx DIV too big"
	ENDIF
; Calculate Tx divider
_TD	SET	INT(0.01 + TXFREQ/F_REF )
;	MESSAGE "TXFREQ " TXFREQ "DIV" TD
TN0	SET	_TD#M
TD1	SET	_TD/M
TN1	SET	TD1#10
TD2	SET	TD1/10
TN2	SET	TD2#10
TD3	SET	TD2/10
TN3	SET	TD3#10
TN4	SET	TD3/10
	IF	TN4>9
	ERROR	"Tx DIV too big"
	ENDIF
; Multiplex the nibbles and make table
	DB	(RN4<<4) | TN4
	DB	(RN3<<4) | TN3
	DB	(RN2<<4) | TN2
	DB	(RN1<<4) | TN1
	DB	(RN0<<4) | TN0
	ENDM
;
; F5 is needed to set frequency REPEAT_CHANNEL.
; F6 is used for frequency/mode channel table entries.
;
F6	MACRO	TXFREQ, RXFREQ, TRMODE
        F5	TXFREQ, RXFREQ
        DB      TRMODE
        ENDM
;
; Something for unused channels
;
F6BLANK	MACRO
	DB 0,0,0,0,0,0
        ENDM
;
; mode bytes for standard channel types
;
;
M_SIMPLEX        EQU 01000001B	; simplex, no CTCSS, no timeout
M_DUPLEX         EQU 00000001B	; duplex,  no CTCSS, no timeout
M_SIMPLEX_CTC    EQU 01010000B	; simplex, Tx CTCSS, with timeout
M_DUPLEX_CTC     EQU 00010001B	; duplex,  Tx CTCSS, no timeout
M_ILINK_CTC      EQU 01110000B  ; simplex, Tx CTCSS, Rx CTCSS
;
; M_SIMPLEX_CTC is for accessing repeaters in the usual way.
;               Timeout warns you about repeater timeouts.
; M_DUPLEX_CTC  is to try duplex on a repeater.
;               No timeout as you should hear repeater pips.
; M_ILINK_CTC   is for internet gateway use.
;               Simplex with Rx and Tx CTCSS enabled.
;               Timeout enabled to protect Tx against PC lock-up               
;
; Build4 Updated info:
;
; b0=0 - enable time-out timer
; b0=1 - disable time-out timer
;
; b3=0 - Head CTC not enabled in tx
; b3=1 - Head CTC enabled for tx
;
; b4=0 - Radio CTC always decodes (doesn't go to encode in tx)
; b4=1 - Radio CTC switches to encode in tx (and can't decode)
;
; b5=0 - Radio CTC monitor is always asserted so Rx CTC is disabled.
; b5=1 - Radio CTC monitor is only asserted if SQ pressed. 
;
; b6=1 - Simplex (Mute Rx in Tx)
; b6=0 - Duplex (don't mute Rx in Tx - but loudspeaker muted)
;
; RC640 Head CTCSS chip is FX315 encode-only device.
; RC690 Head has no CTCSS chip (if used with an RC640 radio)
; RC640 Radio CTCSS chip is FX335 simplex encode/decode chip.
;
; If simultaneous CTCSS decode and encode required in duplex or
; repeater mode, both chips are needed. Radio CTC must decode-only
; and Head must encode.
;
; It makes no sense to set duplex if the Tx and Rx frequencies are
; the same. Tx synth leakage will degrade the Rx sensitivity.
; Simplex will disable the Tx synth in Rx avoiding any problem.
;
; =================================================================
;
CHANMAP:	; start of channel map

; Last channel is where the radio stores the current channel in
; case there is a power-down and retrieves it again at power-up.
; All other radio panel settings are volatile.

; If the radio retrieves an invalid channel it gets a bit lost
; as it never does the stuff that it would do for a new valid
; channel. eg. it will never display a channel number. As zero &
; FFh are invalid, blank RAM certainly would cause this.

;LAST_CHANNEL    location 0, 1 byte.  retains channel during power-off.

                 DB      1               ; 1st valid channel is 1

; This defines the "repeat" channel when in REP  REPh mode
; The "main" channel (presumably) is that currently selected.
;
; So "main" is the mobile user's normal frequency to HQ and
; "repeat" is a frequency used for the local "cell" created
; by the repeater.

;REPEAT_CHANNEL  location 1, 5 bytes. 

                 DB      00h, 00h, 00h, 00h, 00h ; synth data

; The 5-digit selcall code is used to check handhelds accessing the
; repeater function. There is a group-code (0Ah) digit which
; displays as "g" on the control head and (assumption) matches on
; any digit. Selcall uses a 5-tone sequence using EEA call tones.
; The first tone may be longer as it must stop the repeater scanning.
;
; Build 4 New info (from RC690):
;
; The upper nibble of SELCALL contains the GROUP CALL.
; The lower nibble contains INDIVIDUAL CALL.
;
;
;SELCALL_CODE    location 6, 5 bytes. 

                 DB      1,2,3,4,5       ; test value

; If the channel elects to use transmit timeouts then this value
; defines the time. A few seconds before expiry a short bleep
; should sound. A long bleep sounds after expiry and Tx stops.
; max value 254. 255 may stop the timer - don't use

;TOT_TIME        location 11, 1 byte, time out time in seconds.
;
                 DB      180		; 3 minutes
;
; The repeater mode involves scanning the two frequencies "repeat"
; and "main". On detecting a signal it stops and goes to transmit,
; relaying the signal between "repeat" and "main" in the right direction.
; Note there is a 40ms "lock time" - set 100ms and only 60ms is
; actually spent listening.

;REPEAT_TIME     location 12, 1 byte, dwell time on repeat frequency in 10ms

                 DB      10    ; 100ms

;MAIN_TIME       location 13, 1 byte, dwell time on main frequency in 10ms

                 DB      90    ; 900ms
;
;
;
;
; =================================================================
;
; This is it folks! The main channel table.
; The first channel is always channel 01; channel 00 is disallowed.
; Each entry is 6 bytes, 5 frequency bytes and a mode byte.
;
; The mode byte enables selection of:
; b0 Tx timeout timer (on/off)
; b3 Control-head CTCSS (on/off)
; b4 Radio-unit CTCSS (decode/encode)
; b5 Radio-unit CTCSS (monitor/normal)
; b6 Simplex/duplex (duplex = don't mute receiver in Tx)
;
;
;CHANNEL_TABLE   start location 14, 6 bytes per channel, max 99 channels.
; It /was/ 99 channels, firmware mods now allow 255.
; NB channels 1 to 9 must be entered as 01 to 09.
;
;CH.      TXFREQ	  RXFREQ   	MODE				NOTE
;
; Simplex channels 12.5 kHz spacing
;
CH01	F6	144.7750, 144.7700, M_SIMPLEX	; Raynet
CH02	F6	144.7875, 144.7875, M_SIMPLEX	; 
CH03	F6	144.8000, 144.8000, M_SIMPLEX	; 
CH04	F6	144.8125, 144.8125, M_SIMPLEX	; 
CH05	F6	144.8250, 144.8250, M_SIMPLEX	; 
CH06	F6	144.8375, 144.8375, M_SIMPLEX	; 
CH07	F6	144.8500, 144.8500, M_SIMPLEX	; 
CH08	F6	144.8625, 144.8625, M_SIMPLEX	; 
CH09	F6	144.8750, 144.8750, M_SIMPLEX	; 
CH10	F6	144.8875, 144.8875, M_SIMPLEX	; 
CH11	F6	144.9000, 144.9000, M_SIMPLEX	; Packet
CH12	F6	144.9125, 144.9125, M_SIMPLEX	; 
CH13	F6	144.9250, 144.9250, M_SIMPLEX	; 
CH14	F6	144.9375, 144.9375, M_SIMPLEX	; 
CH15	F6	144.9500, 144.9500, M_SIMPLEX	; 
;
; Simplex channels 12.5 kHz spacing
;
CH16	F6	145.2000, 145.2000, M_SIMPLEX	; V16 S8 Emerg Pri
CH17	F6	145.2125, 145.2125, M_SIMPLEX	; V17
CH18	F6	145.2250, 145.2250, M_SIMPLEX	; V18 S9 Emerg Pri
CH19	F6	145.2375, 145.2375, M_SIMPLEX	; V19
CH20	F6	145.2500, 145.2500, M_SIMPLEX	; V20 S10
CH21	F6	145.2625, 145.2625, M_SIMPLEX	; V21
CH22	F6	145.2750, 145.2750, M_SIMPLEX	; V22 S11
CH23	F6	145.2875, 145.2875, M_SIMPLEX	; V23
CH24	F6	145.3000, 145.3000, M_SIMPLEX	; V24 S12
CH25	F6	145.3125, 145.3125, M_SIMPLEX	; V25
CH26	F6	145.3250, 145.3250, M_SIMPLEX	; V26 S13 CARS net
CH27	F6	145.3375, 145.3375, M_SIMPLEX	; V27
CH28	F6	145.3500, 145.3500, M_SIMPLEX	; V28 S14
CH29	F6	145.3625, 145.3625, M_SIMPLEX	; V29
CH30	F6	145.3750, 145.3750, M_SIMPLEX	; V30 S15
CH31	F6	145.3875, 145.3875, M_SIMPLEX	; V31
CH32	F6	145.4000, 145.4000, M_SIMPLEX	; V32 S16
CH33	F6	145.4125, 145.4125, M_SIMPLEX	; V33
CH34	F6	145.4250, 145.4250, M_SIMPLEX	; V34 S17
CH35	F6	145.4375, 145.4375, M_SIMPLEX	; V35
CH36	F6	145.4500, 145.4500, M_SIMPLEX	; V36 S18
CH37	F6	145.4625, 145.4625, M_SIMPLEX	; V37
CH38	F6	145.4750, 145.4750, M_SIMPLEX	; V38 S19
CH39	F6	145.4875, 145.4875, M_SIMPLEX	; V39
CH40	F6	145.5000, 145.5000, M_SIMPLEX	; V40 S20 FM Call
CH41	F6	145.5125, 145.5125, M_SIMPLEX	; V41
CH42	F6	145.5250, 145.5250, M_SIMPLEX	; V42 S21 GB2RS
CH43	F6	145.5375, 145.5375, M_SIMPLEX	; V43 
CH44	F6	145.5500, 145.5500, M_SIMPLEX	; V44 S22 Rally
CH45	F6	145.5625, 145.5625, M_SIMPLEX	; V45
CH46	F6	145.5750, 145.5750, M_SIMPLEX	; V46 S23
CH47	F6	145.5875, 145.5875, M_SIMPLEX	; V47
;
; Repeater channels 12.5 kHz spacing
;
CH48	F6	145.0000, 145.6000, M_SIMPLEX_CTC	; 
CH49	F6	145.0125, 145.6125, M_SIMPLEX_CTC	; 
CH50	F6	145.0250, 145.6250, M_SIMPLEX_CTC	; 
CH51	F6	145.0375, 145.6375, M_SIMPLEX_CTC	; 
CH52	F6	145.0500, 145.6500, M_SIMPLEX_CTC	; 
CH53	F6	145.0625, 145.6625, M_SIMPLEX_CTC	; 
CH54	F6	145.0750, 145.6750, M_SIMPLEX_CTC	; 
CH55	F6	145.0875, 145.6875, M_SIMPLEX_CTC	; 
CH56	F6	145.1000, 145.7000, M_SIMPLEX_CTC	; RV56 GB3KN Maidstone
CH57	F6	145.1125, 145.7125, M_SIMPLEX_CTC	; 
CH58	F6	145.1250, 145.7250, M_SIMPLEX_CTC	; 
CH59	F6	145.1375, 145.7375, M_SIMPLEX_CTC	; 
CH60	F6	145.1500, 145.7500, M_SIMPLEX_CTC	; 
CH61	F6	145.1625, 145.7625, M_SIMPLEX_CTC	; 
CH62	F6	145.1750, 145.7750, M_SIMPLEX_CTC	; 
CH63	F6	145.1875, 145.7875, M_SIMPLEX_CTC	; 
CH64	F6	145.2000, 145.8000, M_DUPLEX		; NA1SS space
;
; Spares
;
CH65    F6BLANK
CH66    F6BLANK
CH67    F6BLANK
;
; Reverse repeaters 12.5 kHz spacing
;
CH68	F6	145.6000, 145.0000, M_SIMPLEX_CTC	; 
CH69	F6	145.6125, 145.0125, M_SIMPLEX_CTC	; 
CH70	F6	145.6250, 145.0250, M_SIMPLEX_CTC	; 
CH71	F6	145.6375, 145.0375, M_SIMPLEX_CTC	; 
CH72	F6	145.6500, 145.0500, M_SIMPLEX_CTC	; 
CH73	F6	145.6625, 145.0625, M_SIMPLEX_CTC	; 
CH74	F6	145.6750, 145.0750, M_SIMPLEX_CTC	; 
CH75	F6	145.6875, 145.0875, M_SIMPLEX_CTC	; 
CH76	F6	145.7000, 145.1000, M_SIMPLEX_CTC	; RV56 GB3KN Maidstone
CH77	F6	145.7125, 145.1125, M_SIMPLEX_CTC	; 
CH78	F6	145.7250, 145.1250, M_SIMPLEX_CTC	; 
CH79	F6	145.7375, 145.1375, M_SIMPLEX_CTC	; 
CH80	F6	145.7500, 145.1500, M_SIMPLEX_CTC	; 
CH81	F6	145.7625, 145.1625, M_SIMPLEX_CTC	; 
CH82	F6	145.7750, 145.1750, M_SIMPLEX_CTC	; 
CH83	F6	145.7875, 145.1875, M_SIMPLEX_CTC	; 
CH84	F6	145.8000, 145.2000, M_DUPLEX		; NA1SS space
;
; Spares
;
CH85	F6BLANK
CH86	F6BLANK
CH87	F6BLANK
CH88	F6BLANK
CH89	F6BLANK
;
; Internet-linking NoV frequencies
;
CH90	F6	144.8125, 144.8125, M_ILINK_CTC	;
CH91	F6	144.8250, 144.8250, M_ILINK_CTC	;  
CH92	F6	144.8375, 144.8375, M_ILINK_CTC	; 
CH93	F6	145.2125, 145.2125, M_ILINK_CTC	; (V17)
CH94	F6	145.2375, 145.2375, M_ILINK_CTC	; (V19)
CH95	F6	145.2875, 145.2875, M_ILINK_CTC	; (V23)
CH96	F6	145.3375, 145.3375, M_ILINK_CTC	; (V27)
;
; Spares
;
	REPT (200-97)
        F6BLANK
        ENDM
;
; Test channels for alignment
;
CH200   F6	130.0000, 130.0000, M_SIMPLEX	; should be out-of-lock
CH201	F6	143.0000, 143.0000, M_SIMPLEX	; Test RC640 bottom limit
CH202	F6	144.0000, 144.0000, M_SIMPLEX	; Test bottom 2m band
CH203	F6	145.0000, 145.0000, M_SIMPLEX	; Test middle 2m band
CH204	F6	146.0000, 146.0000, M_SIMPLEX	; Test top    2m band
CH205	F6	148.0000, 148.0000, M_SIMPLEX	; Test top    USA 2m?
CH206   F6	149.0000, 149.0000, M_SIMPLEX	; Test RC640 centre
CH207	F6	156.0000, 156.0000, M_SIMPLEX	; Test RC640 top limit
CH208	F6	162.0000, 162.0000, M_SIMPLEX	; Test ??? top limit
CH209   F6	170.0000, 170.0000, M_SIMPLEX	; should be out-of-lock
;
; RC640 normally kills the Tx synth when in Rx. The Tx synth running on
; the Rx freq may leak into the Rx. The duplex flag leaves it on.
; This is useful as you don't have to PTT to test the Tx synth.
;
CH210   F6	130.0000, 130.0000, M_DUPLEX	; should be out-of-lock
CH211	F6	143.0000, 143.0000, M_DUPLEX	; Test RC640 bottom limit
CH212	F6	144.0000, 144.0000, M_DUPLEX	; Test bottom 2m band
CH213	F6	145.0000, 145.0000, M_DUPLEX	; Test middle 2m band
CH214	F6	146.0000, 146.0000, M_DUPLEX	; Test top    2m band
CH215	F6	148.0000, 148.0000, M_DUPLEX	; Test top    USA 2m?
CH216   F6	149.0000, 149.0000, M_DUPLEX	; Test RC640 centre
CH217	F6	156.0000, 156.0000, M_DUPLEX	; Test RC640 top limit
CH218	F6	162.0000, 162.0000, M_DUPLEX	; Test ??? top limit
CH219   F6	170.0000, 170.0000, M_DUPLEX	; should be out-of-lock
;
; When working on a Tx synth, it may be easier to fit it in place of
; the Rx synth. No PA on the lid. In that case use these channels
;
CH220   F6	130-10.7, 130+10.7, M_DUPLEX	; should be out-of-lock
CH221	F6	143-10.7, 143+10.7, M_DUPLEX	; Test RC640 bottom limit
CH222	F6	144-10.7, 144+10.7, M_DUPLEX	; Test bottom 2m band
CH223	F6	145-10.7, 145+10.7, M_DUPLEX	; Test middle 2m band
CH224	F6	146-10.7, 146+10.7, M_DUPLEX	; Test top    2m band
CH225	F6	148-10.7, 148+10.7, M_DUPLEX	; Test top    USA 2m?
CH226   F6	149-10.7, 149+10.7, M_DUPLEX	; Test RC640 centre
CH227	F6	156-10.7, 156+10.7, M_DUPLEX	; Test RC640 top limit
CH228	F6	162-10.7, 162+10.7, M_DUPLEX	; Test ??? top limit
CH229   F6	170-10.7, 170+10.7, M_DUPLEX	; should be out-of-lock
;
	REPT (255-229)
        F6BLANK
        ENDM
;
END_CHANMAP     DB 0	; label for end of channel map
;
; =================================================================
; Documentation and Analysis follows
;
;Channel data
;
; MODE byte 
;  .......1    Time out timer      :- 1 = disabled     0 = enabled
;  ......1.    not used.
;  .....1..    not used (was Low transmit power select).
;  ....1...    Control unit CTCSS  :- 1 = selected on  0 = off
;  ...1....    Radio CTCSS unit    :- 1 = decode       0 = encode 
;  ..1.....    Radio CTCSS monitor :- 1 = off          0 = on 
;  .1......    frequency operation :- 1 = simplex      0 = duplex 
;  1.......    not used (was Reverse frequency operation).
;
; b5=0 forces Radio CTCSS monitor on (ie. defeats CTCSS)
; b5=1 enables Radio tone squelch
;
; SYNTHESISER 5 bytes
;    ....1111    Transmit synthesiser data (LS nibble). 
;    1111....    Receive synthesiser data (MS nibble). 
;
; The CPU drives HEF8751V /A0-/A3 via 7406 hex inverting buffers;
; this means it must be fed true data rather than inverted.
; Data actually strobed into synthesisers is as follows:
;
; 88h		Fetch 6    not loaded
; main_freq+4   Fetch 0    load n0
; main_freq+3   Fetch 1    load n1
; main_freq+2   Fetch 2    load n2
; main_freq+1   Fetch 3    load n3
; main_freq+0   Fetch 4    load n4
; 00h           Fetch 5    load n5
; 88h           Fetch 6    load M (nb 0->16)
;
; Fetch 1-5 /B3=1 /B2=1 /B1=1 /B0=1 (n0-n5 = 0)
; Fetch 6   /B3=1 /B2=1 /B1=0 /B0=0 (C0b, H, nh)
;
;           /B3=1 /B2=1  ->  C0b=10/11
;           /B1=0 /B0=0  ->  H=1 (no half-channel)
;
; From this we conclude  M=8 and n5=0 (fixed in firmware).
; The RAM table data is in the order n4 n3 n2 n1 n0, MS nibble
; in lowest address.
;
;
; The synth comparison frequency is 50kHz derived from 10MHz by
; dividing by 200. This is set by the pin-strapping of the HEF8750V
; reference divider. This frequency gets compared with the OFF output
; from the HEF8751V and is "M" times the final channel step. When
; locked the phase comparator is "strobed" by OFS and runs at a
; submultiple of 50kHz; ie 50kHz/M.
;
; This mess is to do with a desire to get fast locking, necessary?
;
; The HEF8751V has an SP8690 divide 10/11 prescaler.
;
; The final quirk is that the PLL loop VCO does not run at the
; final synth output frequency. It is followed by a x2 frequency
; multiplier.
; 
; The result of this is:
; -> the loop itself runs at half the wanted LO frequency.
; -> the PLL step is half the LO step (12.5kHz/2 = 6.25kHz)
; -> the output of the VCO divider chain is 50kHz/8 = 6.25kHz
;
; Anyhow, We can pretend the final x2 doesn't exist and assume
; 12.5kHz steps and we get the right divider values.
;
; Calculating the synth numbers:
;
; Let nibbles n4, n3, n2, n1, n0 be what we're trying to find.
; 
; We calculate required LO frequencies:
; Tx LO equals RF freq
; Rx LO equals RF freq -10.7MHz
;
; We calculate the required D as LO / 12.5kHz
;
; Division D = (n5*10000 + n4*1000 + n3*100 + n2*10 + n1)*M + n0
;
; Calculate n0 as D mod M.      (remember M=8)
; Calculate D1 as int( D / M )
;
; Split D1 into four BCD digits n4:n3:n2:n1  (n5=0 fixed)
;
; We can calculate the decimal digits by:
; n1 = D1 mod 10
; D2 = int( D1 / 10 )
; ...and continuing for each more significant digit.
;
; Data for Rx synth and Tx synth must be merged with
; Rx synth in high nibble and Tx synth in low nibble.
;
;The end
;